import { world, system, TimeOfDay } from "@minecraft/server";
import * as Vector3 from "./vector3";


let scoreboard = world.scoreboard;
export const overworld = world.getDimension('overworld');
export const nether = world.getDimension('nether');
export const the_end = world.getDimension('the_end');
export const dimensions = [overworld, nether, the_end];

export let host = undefined;


export function randomInt(min, max) {
	return Math.floor(Math.random() * (max - min + 1))+min;
}

export async function waitForTicks(ticks) {
    return await system.runTimeout(stam, ticks);
}

export async function waitForLoopEvent(ticks, callback) {
    return new Promise(resolve => {
        let id = 0;
        const event = async function () {
            var _a;
            if (await callback()) {
                resolve();
            }
            else {
                id = system.runTimeout(event, ticks);
            }
        };
        system.run(event);
    });
}

export async function loadHost() {
	if (host)
		return;
	
	await waitForLoopEvent(1, () => {
		return world.getAllPlayers().length > 0;
	});
	host = await world.getAllPlayers()[0];
	if (!host.hasTag("tcs_cba_isHost"))
		host.addTag("tcs_cba_isHost");
}



// This function replaces the old initBedrock function
export async function onLoadFirstTime()
{
	// If needed - initialize scoreboard function, if not - comment this
	//await runCommand("function tcs/tad/params/create_objectives");
	//await runCommand("function tcs/tad/params/set_scores");
}


export async function runCommand(command) {
	return await overworld.runCommandAsync(command);
}

export async function runHostCommand(command) {
	return await host.runCommandAsync(command);
}

export async function runPlayerCommand(entity, command) {
	return await entity.runCommandAsync(command);
}
export async function runDelayedPlayerCommand(delay, entity, command) {
	return await system.runTimeout(() => { runPlayerCommand(entity, command); }, delay);
}
export async function runMobCommand(entity, command) {
	return await entity.runCommandAsync(command);
}

export async function getEntity(entityType)
{
	const entities = await dimensions.flatMap((dim) => (dim.getEntities({ type: entityType })));
	if (entities !== undefined && entities.length > 0)
		return entities[0];
	return undefined;
}
export async function getEntities(entityType)
{
	const entities = await dimensions.flatMap((dim) => (dim.getEntities({ type: entityType })));
	return entities;
}
export async function getEntityNearEntity(entityType, nearbyEntity)
{
	const entities = await nearbyEntity.dimension.getEntities( { type: entityType, location: nearbyEntity.location, closest: 1 } );
	return entities;
}
export async function getEntityInLocation(entityType, dim, centerLocation)
{
	const entities = await dim.getEntities( { type: entityType, location: centerLocation, closest: 1 } );
	if (entities !== undefined && entities.length > 0)
		return entities[0];
	return undefined;
}
export async function getPlayerInLocation(dim, centerLocation)
{
	const players = await dim.getPlayers( { location: centerLocation, closest: 1 } );
	return players;
}

export async function getEntitiesInArea(entityType, dim, corner1, corner2, checkDimensions, tag)
{
	// checkDimensions - what dimensions to check - "x", "xz", "xyz", etc
	let entities = await dim.getEntities( { type: entityType } );
	if (tag !== undefined)
		entities = entities.filter( (entity) =>  entity.hasTag(tag));
	return entities.filter( (entity) =>  Vector3.locationIsInArea(entity.location, corner1, corner2, checkDimensions));
}


export async function showPlayerErrorMessage(player, message)
{
	if (player !== undefined)
	{
		await runPlayerCommand(player, "title @s actionbar "+message);
		await player.playSound("tcs_cba:no_button", { volume: 0.8 } );
	}
	else
	{
		await runCommand("title @a actionbar "+message);
		await runCommand("playsound tcs_cba:no_button @a ~ ~ ~ 100 1 0.8");
	}
}


export async function tellraw(entity, target, msg) {
	return await entity.runCommandAsync(`tellraw ${target} {"rawtext":[{"text":"§r${msg}"}]}`);
}

export async function title(player, target, msg) {
	await runPlayerCommand(player, `title ${target} times 20 60 20`);
	return await player.runCommandAsync(`titleraw ${target} title {"rawtext":[{"text":"§r${msg}"}]}`);
}
export async function subtitle(player, target, msg, addEmptyTitle=false) {
	await runPlayerCommand(player, `title ${target} times 20 60 20`);
	await player.runCommandAsync(`title ${target} subtitle ${msg}`);
	if (addEmptyTitle === true)
		await player.runCommandAsync(`title ${target} title `);
}
export async function actionbar(player, target, msg) {
	await runPlayerCommand(player, `title ${target} times 20 60 20`);
	return await player.runCommandAsync(`titleraw ${target} actionbar {"rawtext":[{"text":"§r${msg}"}]}`);
}

export async function playSound(player, target, playSound, volume, pitch, minVolume) {
	let soundVolume = volume ? volume : 100;
	let soundPitch = pitch ? pitch : 1;
	let soundMinVolume = minVolume ? minVolume : 1;
	return await player.runCommandAsync(`playsound ${playSound} ${target} ~ ~ ~ ${soundVolume} ${soundPitch} ${soundMinVolume}`);
}

export function debug(message) {
	return world.sendMessage(message);
}


export async function verifyAir2(corner1, corner2)
{
	for (let x=corner1.x; x<=corner2.x; x++)
	{
		for (let y=corner1.y; y<=corner2.y; y++)
		{
			for (let z=corner1.z; z<=corner2.z; z++)
			{
				let block = await overworld.getBlock({x: x, y: y, z: z});
				if (!isBlockAir(block))
					return false;
			}
		}
	}
	return true;
}
const ignoreBlocks = [ 'air', 'light_block', 'white_carpet', 'orange_carpet', 'magenta_carpet',
	'light_blue_carpet', 'yellow_carpet', 'lime_carpet', 'pink_carpet', 'gray_carpet', 'light_gray_carpet',
	'cyan_carpet', 'purple_carpet', 'blue_carpet', 'brown_carpet','green_carpet', 'red_carpet', 'black_carpet',
	'grass', 'short_grass', 'tall_grass', 'blue_orchid', 'allium', 'azure_bluet', 'red_tulip', 'orange_tulip', 'white_tulip',
	'pink_tulip', 'oxeye_daisy', 'cornflower', 'lily_of_the_valley', 'wither_rose', 'fern', 'large_fern','snow_layer', 'dandelion',
	'poppy', 'allium', 'sunflower', 'lilac', 'rose_bush', 'peony', 'brown_mushroom' ];

export function isBlockAir(block)
{
	if (block === null || block === undefined || block.isAir == undefined || block.isAir) 
		return true;
	if (ignoreBlocks.includes(block.typeId.replace("minecraft:","")))
		return true;
	return false;
}
